@@ -1,7 +1,7 @@
 use super::*;
 use crate::{gl46 as native_gl, version::Version};
-use std::{collections::HashSet, ffi::CString, num::NonZeroU32};
 use std::ffi::CStr;
+use std::{collections::HashSet, ffi::CString, num::NonZeroU32};
 
 #[derive(Default)]
 struct Constants {
@@ -12,6 +12,7 @@ pub struct Context {
     raw: native_gl::GlFns,
     extensions: HashSet<String>,
     constants: Constants,
+    version: Version,
 }
 
 impl Context {
@@ -30,19 +31,25 @@ impl Context {
                 loader_function(c_str.to_str().unwrap()) as *mut std::os::raw::c_void
             });
 
+        // Retrieve and parse `GL_VERSION`
+        let raw_string = raw.GetString(VERSION);
+        let raw_version = std::ffi::CStr::from_ptr(raw_string as *const native_gl::GLchar)
+            .to_str()
+            .unwrap()
+            .to_owned();
+        let version = Version::parse(&raw_version).unwrap();
+
         // Setup extensions and constants after the context has been built
         let mut context = Self {
             raw,
             extensions: HashSet::new(),
             constants: Constants::default(),
+            version,
         };
 
-        let raw_version = context.get_parameter_string(VERSION);
-        let version = Version::parse(&raw_version).unwrap();
-
         // Use core-only functions to populate extension list
-        if (version >= Version::new(3, 0, None, String::from("")))
-            || (version >= Version::new_embedded(3, 0, String::from("")))
+        if (context.version >= Version::new(3, 0, None, String::from("")))
+            || (context.version >= Version::new_embedded(3, 0, String::from("")))
         {
             let num_extensions = context.get_parameter_i32(NUM_EXTENSIONS);
             for i in 0..num_extensions {
@@ -139,6 +146,10 @@ impl HasContext for Context {
         self.extensions.contains("GL_KHR_debug")
     }
 
+    fn version(&self) -> &Version {
+        &self.version
+    }
+
     unsafe fn create_framebuffer(&self) -> Result<Self::Framebuffer, String> {
         let gl = &self.raw;
         let mut name = 0;
@@ -2648,16 +2659,11 @@ impl HasContext for Context {
         &self,
         program: Self::Program,
         uniform_block_index: u32,
-        parameter: u32
+        parameter: u32,
     ) -> i32 {
         let gl = &self.raw;
         let mut value = 0;
-        gl.GetActiveUniformBlockiv(
-            program.0.get(),
-            uniform_block_index,
-            parameter,
-            &mut value
-        );
+        gl.GetActiveUniformBlockiv(program.0.get(), uniform_block_index, parameter, &mut value);
         value
     }
 
@@ -2666,20 +2672,20 @@ impl HasContext for Context {
         program: Self::Program,
         uniform_block_index: u32,
         parameter: u32,
-        out: &mut [i32]
+        out: &mut [i32],
     ) {
         let gl = &self.raw;
         gl.GetActiveUniformBlockiv(
             program.0.get(),
             uniform_block_index,
             parameter,
-            out.as_mut_ptr()
+            out.as_mut_ptr(),
         );
     }
     unsafe fn get_active_uniform_block_name(
         &self,
         program: Self::Program,
-        uniform_block_index: u32
+        uniform_block_index: u32,
     ) -> String {
         let gl = &self.raw;
 
@@ -2690,7 +2696,8 @@ impl HasContext for Context {
         let len = self.get_active_uniform_block_parameter_i32(
             program,
             uniform_block_index,
-            crate::UNIFORM_BLOCK_NAME_LENGTH);
+            crate::UNIFORM_BLOCK_NAME_LENGTH,
+        );
         let len = if gl.GetError() == crate::NO_ERROR && len > 0 {
             len as usize
         } else {
@@ -2712,15 +2719,18 @@ impl HasContext for Context {
                 std::mem::size_of::<u8>(),
                 std::mem::size_of::<native_gl::GLchar>(),
                 "This operation is only safe in systems in which the length of \
-                a GLchar is the same as that of an u8");
+                a GLchar is the same as that of an u8"
+            );
             assert_eq!(
                 std::mem::align_of::<u8>(),
                 std::mem::align_of::<native_gl::GLchar>(),
                 "This operation is only safe in systems in which the alignment \
-                of a GLchar is the same as that of an u8");
+                of a GLchar is the same as that of an u8"
+            );
             let buffer = std::slice::from_raw_parts(
                 buffer.as_ptr() as *const u8,
-                (length as usize + 1).min(buffer.len()));
+                (length as usize + 1).min(buffer.len()),
+            );
 
             let name = CStr::from_bytes_with_nul(&buffer[..])
                 .unwrap()
